'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.rules = undefined;

var _graphql = require('graphql');

var rules = {
  'template-strings': function templateStrings(context) {
    var _context$options$ = context.options[0];
    var schemaJson = _context$options$.schemaJson;
    var _context$options$$tag = _context$options$.tagName;
    var tagName = _context$options$$tag === undefined ? 'gql' : _context$options$$tag;


    if (!schemaJson) {
      throw new Error('Must pass in schemaJson option.');
    }

    var unpackedSchemaJson = schemaJson.data ? schemaJson.data : schemaJson;

    if (!unpackedSchemaJson.__schema) {
      throw new Error('Please pass a valid GraphQL introspection query result.');
    }

    var schema = (0, _graphql.buildClientSchema)(unpackedSchemaJson);

    return {
      TaggedTemplateExpression: function TaggedTemplateExpression(node) {
        if (node.tag.name !== tagName) {
          return;
        }

        if (node.quasi.quasis.length > 1) {
          context.report({
            node: node,
            message: 'Unexpected interpolation in GraphQL template string.'
          });

          return;
        }

        var text = node.quasi.quasis[0].value.cooked;

        var ast = void 0;

        try {
          ast = (0, _graphql.parse)(text);
        } catch (error) {
          context.report({
            node: node,
            message: error.message.split('\n')[0],
            loc: locFrom(node, error)
          });
          return;
        }

        var validationErrors = schema ? (0, _graphql.validate)(schema, ast) : [];

        if (validationErrors && validationErrors.length > 0) {
          context.report({
            node: node,
            message: validationErrors[0].message,
            loc: locFrom(node, validationErrors[0])
          });
          return;
        }
      }
    };
  }
};

function locFrom(node, error) {
  var location = error.locations[0];

  var line = void 0;
  var column = void 0;
  if (location.line === 1) {
    line = node.loc.start.line;
    column = node.loc.start.col + location.col;
  } else {
    line = node.loc.start.line + location.line - 1;
    column = location.column - 1;
  }

  return {
    line: line,
    column: column
  };
}

exports.rules = rules;